/***************************************************************************
                          coordinates.h  -  description
                             -------------------
    begin                : Thu Mar 1 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COORDINATES_H
#define COORDINATES_H

#include <string>

namespace graphlib {

/**
  * @short Coordinates of a node
  */

class Coordinates
{
    //------------------------------------------------------ friend functions
    /**
      * Output-operator.
      * Needed for GML output of the nodes of a graph
      *
      * @param os the output-stream
      * @param c the coordinates
      * @see Node
      * @return the output-stream
      */
    friend ostream& operator<<(ostream& os, const Coordinates& c);

  public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Default Constructor.
      * Creates new coordinates. Default values are for all 3 coordinates
      * 0.0.
      *
      * @param x coordinate
      * @param y coordinate
      * @param z coordinate
      */
    Coordinates(double x=0.0, double y=0.0, double z=0.0);

    //------------------------------------------------------ Operators
    /**
      * Assignement operator.
      * Assigns the coordinates of <code>c</code> to this one.
      *
      * @param c original coordinates
      * @return itself
      */
    Coordinates& operator=(const Coordinates& c);

    /**
      * Assignement operator.
      * Assigns the <code>c</code> to all 3 of this one.
      *
      * @param c original coordinates
      * @return itself
      */
    Coordinates& operator=(double c);

    /**
      * Comparision operator equal.
      * Returns true if all three coordinates are equal to
      * <code>border</code> plus/minus EPSILON else false.
      *
      * @param border
      */
    bool operator==(double border);

    /**
      * Comparision operator less.
      * Returns true if all three coordinates are less then
      * <code>border</code> else false.
      *
      * @param border
      */
    bool operator<(double border);

    /**
      * Comparision operator greater.
      * Returns true if all three coordinates are greater then
      * <code>border</code> else false.
      *
      * @param border
      */
    bool operator>(double border);

    //------------------------------------------------------ datas
    /// x coordinale
    double x;
    /// y coordinate
    double y;
    /// z coordinate
    double z;

}; // class Coordinates

} // namespace graphlib
#endif // COORDINATES_H

/* end of file */
