/***************************************************************************
                          cycle.cpp  -  description                              
                             -------------------                                         
    begin                : Wed Dec 22 1999                                           
    copyright            : (C) 1999 by Gleiss Petra                         
    email                : petra@gleiss.at                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "cycle.h"
#include "trace.h"

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
Cycle::Cycle(unsigned size, const Graph* g) : BitVector(size), SubGraph(g),
                 prototype(0), xchgClass(0), member(0)
{ ; }

Cycle::Cycle(const Cycle& aC) : BasisGraph(aC), BitVector(aC), SubGraph(aC),
                 prototype(aC.prototype), xchgClass(aC.xchgClass),
                 member(aC.member)
{ ; }

//---------------------------------------------------------------------------
// Operators
//---------------------------------------------------------------------------
Cycle& Cycle::operator=(const Cycle& aC)
{
  if (this == &aC) return *this;
  BitVector::operator=(aC);
  SubGraph::operator=(aC);

  prototype = aC.prototype;
  xchgClass = aC.xchgClass;
  member = aC.member;
  return *this;
}

/*bool Cycle::operator==(const Cycle& aC) const
{
  if (size!=aC.size)
    THROW(SizeError,"Cycle::operator== (const Cycle&) const")

  if (count()!=aC.count())
    return false;

  if (index && aC.index && index==aC.index)
    return true;

  for (unsigned i=0; i<words; i++)
    if (buffer[i]!=aC.buffer[i])
      return false;
  return true;
}

bool Cycle::operator!=(const Cycle& aC) const
{
  return !(*this == aC);
} */

bool Cycle::lessIndex(const Cycle& aC) const throw(SizeError)
{
  if (size!=aC.size)
    THROW(SizeError,"Cycle::lessIndex(const Cycle&) const")
  return (number_ < aC.number_);
}
bool Cycle::lessPrototypeIndex(const Cycle& aC) const throw(SizeError)
{
  if (size!=aC.size)
    THROW(SizeError,"Cycle::lessPrototypeIndex(const Cycle&) const")
  return (prototype < aC.prototype);
}
bool Cycle::lessXchgClass(const Cycle& aC) const throw(SizeError)
{
  if (size!=aC.size)
    THROW(SizeError,"Cycle:lessXchgClass(const Cycle&) const")
  return (xchgClass < aC.xchgClass);
}

//---------------------------------------------------------------------------
// I/O
//---------------------------------------------------------------------------
ostream& operator<< (ostream& os, const Cycle& aC)
{
  // cycle
  if (!aC.getmembers(Cycle::UPath))
  {
    os << "# gml undirected Cycle" << endl;
    os << "# " << timeStamp() << endl;
    os << "cycle [" << endl;
  }
  else
  {
    os << "# gml undirected U-Path" << endl;
    os << "# " << timeStamp() << endl;
    os << "path [" << endl;
  }

  os << "  id " << aC.index() << endl;
  if (aC.getmembers(Cycle::Prototype))
    os << "  prototype 1" << endl;
  if (aC.getmembers(Cycle::Mcb))
    os << "  mcb_member 1" << endl;
  if (aC.getmembers(Cycle::Shortest))
    os << "  shortest 1" << endl;
  if (aC.getmembers(Cycle::UniqueShortest))
    os << "  unique_shortest 1" << endl;
  if (aC.getmembers(Cycle::Essential))
    os << "  essential 1" << endl;

  constNodeIter it = aC.cycleNodes().begin();
  constNodeIter end= aC.cycleNodes().end();

  os << "  [\n";
  while (it != end)
    os << "    vertex " << (it++)->id() << endl;

  os << "  ]\n]\n";
  return os;
}

ostream& operator<< (ostream& os, const Cycle* aC)
{
  os << "\t" << aC->index();
  os << "\t" << aC->prototypeIndex();
  os << "\t" << aC->xchgClassIndex() ;
  os << "\t" << aC->count() << "\t";

  aC->write_members(os);

  os << "\t" << aC->nodes;
  os << endl;
  return os;
}

//---------------------------------------------------------------------------
// Information
//---------------------------------------------------------------------------
unsigned long Cycle::prototypeIndex() const
{
  return prototype;
}
void Cycle::prototypeIndex(unsigned long aproto)
{
  prototype = aproto;
}
unsigned long Cycle::xchgClassIndex() const
{
  return xchgClass;
}
void Cycle::xchgClassIndex(unsigned long ainter)
{
  xchgClass = ainter;
}
bool Cycle::getmembers(MemberFlag pos) const
{
  return (member & (0x1 << (7 - pos%8)));
}
void Cycle::setmembers(MemberFlag pos)
{
  member |= (0x1 << (7 - pos%8));
}
void Cycle::clearmembers(MemberFlag pos)
{
  member &= ~(0x1 << (7 - pos%8));
}	

/// additional functions
void Cycle::write_members(ostream& os) const
{
  os << "< ";
  for (int j=0;j<8;j++)
    if (getmembers((MemberFlag)j))
      os << "1 ";
    else
      os << "0 ";

  os << ">";
}

string Cycle::membership() const
{
  string tmp;
  for (int j=0; j<8; j++)
    if (getmembers((MemberFlag)j))
      tmp += "1 ";
    else
      tmp += "0 ";

  return tmp;
}

//---------------------------------------------------------------------------
// Creation
//---------------------------------------------------------------------------
void Cycle::addNodes(const Node& node)
{
  // insert the nodes
  nodes.push_back(node);
}

void Cycle::addNodes(NodeIter begin, NodeIter end)
{
  // insert the nodes
  nodes.insert(nodes.end(),begin,end);
}

void Cycle::addNodes(list<Node>::reverse_iterator begin,
                    list<Node>::reverse_iterator end)
{
  // insert the nodes
  nodes.insert(nodes.end(),begin,end);
}


bool Cycle::calcCycle(const EdgeIdMap& edgeMap) throw(logic_error,runtime_error)
{
  // clears all Bits
  if (length != 0)
    clearAllBits();

  if (nodes.empty())
    THROW(logic_error,"Cycle::calcCycle(const EdgeIdMap&): no nodes set")

  if (nodes.size()==1)
    THROW(logic_error,"Cycle::calcCycle(const EdgeIdMap&): only one node set")

  // first node
  constNodeIter first = nodes.begin();

  // second node
  constNodeIter second = nodes.begin();
  second++;

  constEdgeIter edge;
  unsigned edgePos;

  while (second != nodes.end())
  {
    // find edge between first and second
    if ((edge=first->findEdge(*second)) == first->adjEnd())
    {
      string error = "Cycle::calcCycle(const EdgeIdMap&): no edge between ";
      error += Ttos(first->id());
      error += " and "+Ttos(second->id());
      THROW(runtime_error,error)
    }

    // number of the edge
    edgePos = edgeMap.find(*edge)->second;

    // set edge
    edges.push_back(*edge);

    // set bit with pos = number of the edge
    if (setBit(edgePos) == false)
      return false;

    first++; second++;
  }

  return true;
}

} // namespace graphlib

/* end of file */
