/***************************************************************************
                          dot2gml.cpp  -  description
                             -------------------
    begin                : Sat Jul 28 2001
    copyright            : (C) 2001 by Petra Gleiss
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dot2gml.h"
#include "utilities.h"
#include "trace.h"


namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
Dot2Gml::~Dot2Gml()
{
  if (in != NULL)
  {
    in->close();
    delete in;
  }
}

void Dot2Gml::init(string fname) throw(FileError)
{
  in = new ifstream(fname.c_str(), ios::in);

  if (in->bad() || in->fail())
    THROW(FileError,("Can't open dot file "+fname+".").c_str())
}

//---------------------------------------------------------------------------
// Methods
//---------------------------------------------------------------------------
void Dot2Gml::dot2gml(Graph& g) throw(FileError,SyntaxError)
{
  g.clear();

  // reads file -> string
  readStructure();

  // structure -> graph
  makeGraph(g);

  // minCoord = inf ; maxCoord = -inf.
  g.resetMinMaxCoord();
}

void Dot2Gml::readStructure() throw(FileError)
{
  if (in == 0)
    THROW(FileError,"No file opened!")

  getline(*in,structure);

}

void Dot2Gml::makeGraph(Graph& g) throw(SyntaxError)
{
  unsigned i=0, length=structure.length();
  vector<NodeIter> rund, eckig, geschwungen, zackig;
  NodeIter now, prev;

  for (i=0; i!=length; i++)
  {
    // this node
    now = g.newNode();

    switch(structure[i])
    {
      // not paired node
      case '.':
        break;

      case '(':
        rund.push_back(now);
      break;

     case ')':
       if (rund.empty())
         THROW(SyntaxError,"unbalanced ()-brackets - "+Ttos(i))
       g.newEdge(*now,*rund.back());
       cout << *rund.back() << " " << *now << "upper\n";
       rund.pop_back();
       break;

     case '[':
       eckig.push_back(now);
       break;

     case ']':
       if (eckig.empty())
         THROW(SyntaxError,"unbalanced []-brackets - "+Ttos(i))
       g.newEdge(*now,*eckig.back());
       cout << *eckig.back() << " " << *now << "lower\n";
       eckig.pop_back();
       break;

     case '{':
       geschwungen.push_back(now);
       break;
     case '}':
       if (geschwungen.empty())
         THROW(SyntaxError,"unbalanced {}-brackets - "+Ttos(i))
       g.newEdge(*now,*geschwungen.back());
       cout << *geschwungen.back() << " " << *now << "{}\n";
       geschwungen.pop_back();
       break;

     case '<':
       zackig.push_back(now);
       break;
     case '>':
       if (zackig.empty())
         THROW(SyntaxError,"unbalanced <>-brackets - "+Ttos(i))
       g.newEdge(*now,*zackig.back());
       cout << *zackig.back() << " " << *now << "<>\n";
       zackig.pop_back();
       break;

      // may not happen !!!
      default:
        THROW(SyntaxError,"undefined symbol nr "+Ttos(i)+": "+Ttos(structure[i]))
    }

    // backbone: edge(prev,now)
    if (i!=0)
      g.newEdge(*prev,*now);

    // prev <- this node
    prev = now;
  }

  if (!rund.empty())
    THROW(SyntaxError,"unbalanced ()-brackets: open")
  if (!eckig.empty())
    THROW(SyntaxError,"unbalanced []-brackets: open")
  if (!geschwungen.empty())
    THROW(SyntaxError,"unbalanced {}-brackets: open")
  if (!zackig.empty())
    THROW(SyntaxError,"unbalanced <>-brackets: open")
}

} //namespace graphlib

/* end of file */
