/***************************************************************************
                          graphlib.h  -  description
                             -------------------
    begin                : Mon Apr 2 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPHLIB_H
#define GRAPHLIB_H

#define GRAPHLIB_TRACE
#define GRAPHLIB_DEBUG 6

#include <climits>
#include <cmath>
#include <string>

namespace graphlib
{
  // sign of numbers
  enum Sign { NEG = -1, ZERO = 0, POS = 1, NONE = 20};

  //
  enum CalcStat { NO = 0, CI, UPATHES, PROTOTYPES, RELEVANT, SHORT,
                  UNIQUES, ESSENT };

  /// Flag for calculation the cycle decomposition
  enum DecompositionsMode { None = 0,
                            Horton,
                            Prototypes,
                            Relevants,
                            UniqueShortests,
                            Essentials,
                            InterchangeabilityClasses,
                            All
                           };

  // Flag for graphs
  enum GraphFlag { Empty = 0,
                   Bipartite,
                   Complete,
                   Hamiltonian,
                   Random
                 };

  // strings for graphs-flags
  static const string GraphFlagString[] = { "Empty",
                                            "Bipartite",
                                            "Complete",
                                            "Hamiltonian",
                                            "Random"
                                          };

  /// precision for calculating
  static const double EPSILON = 0.000000000001;
  /// infinity
  static const unsigned INF = UINT_MAX;
  /// infinity
  static const int INFINITY = INT_MAX;

  /// maximal number of nodes
  static const unsigned MAX_NODES = 10000;

  class Trace;
  class LogFile;

  class BasisGraph;
  class Graph;
  class SubGraph;
  class CycleGraph;
  class RandomGraph;
  class RandCycleGraph;
  class Dot2Gml;

  class BiComponents;
  class Eigenvector_Eigenvalue;
  class NodalDomain;

  class BitVector;
  class Cycle;
  class Prototype;

} // namespace graphlib

#endif // GRAPHLIB_H

/* end of file */
