/***************************************************************************
                          doubleerror.h  -  description
                             -------------------
    begin                : Mon Apr 2 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPHLIB_ERROR_H
#define GRAPHLIB_ERROR_H

#include <stdexcept>
#include <string>

namespace graphlibError {

/**
  *@short Error thrown by graphlib
  */

class DoubleError : public runtime_error
{
  public:

    DoubleError(const string& what_arg)
        : runtime_error(what_arg) {;}
}; // class DoubleError

class FileError : public runtime_error
{
  public:

    FileError(const string& what_arg)
       : runtime_error(what_arg) {;}
}; // class FileError

class SyntaxError : public runtime_error
{
  public:

    SyntaxError(const string& what_arg)
       : runtime_error(what_arg) {;}
}; // class SyntaxError




class SizeError : public logic_error
{
  public:
    SizeError(const string& what_arg)
       : logic_error(what_arg+": different size") {;}
}; // class SizeError

class OutsideError : public logic_error
{
  public:
    OutsideError(const string& what_arg)
       : logic_error(what_arg) {;}
}; // class OutsideError

class BufferError : public logic_error
{
  public:
    BufferError(const string& what_arg)
       : logic_error(what_arg+": no buffer set") {;}
}; // class BufferError

class EmptyCycleError : public logic_error
{
  public:
    EmptyCycleError(const string& what_arg)
       : logic_error(what_arg) {;}
}; // class EmptyCycleError

class EmptyEdgeError : public logic_error
{
  public:
    EmptyEdgeError(const string& what_arg)
       : logic_error(what_arg+": Edge without data") {;}
}; // class EmptyEdgeError

class EmptyGraphError : public logic_error
{
  public:
    EmptyGraphError(const string& what_arg)
       : logic_error(what_arg+": empty Graph") {;}
}; // class EmptyGraphError

class EmptyMatrixError : public logic_error
{
  public:
    EmptyMatrixError(const string& what_arg)
       : logic_error(what_arg+": Matrix without data") {;}
}; // class EmptyMatrixError

class EmptyNodeError : public logic_error
{
  public:
    EmptyNodeError(const string& what_arg)
       : logic_error(what_arg+": Node without data") {;}
}; // class EmptyNodeError


class Error : public logic_error
{
  public:
    Error(const string& what_arg)
       : logic_error(what_arg) {;}
}; // class Error

} // namespace graphlibError

#endif //GRAPHLIB_ERROR_H

/* end of file */
