/***************************************************************************
                          graphlib_utilities.h  -  description
                             -------------------
    begin                : Wed Jun 6 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPHLIB_UTILITIES_H
#define GRAPHLIB_UTILITIES_H

#include <string>
#include <strstream>

#include <list>

namespace graphlib {

  /**
    * Formates <code>arg</code> to: 1.xxxxxxxxxxxxxxxx*^Exponent.
    * (for writeMathematica)
    */
  template <typename T> inline
  string toMathematica(T arg)
  {
    ostrstream buffer;
    buffer.setf(ios::scientific);
    buffer.precision(16);
    buffer << arg << ends;
    string zahl(buffer.str());
    buffer.rdbuf()->freeze(false);
    zahl.replace(zahl.find("e"),1,"*^");
    return zahl;
  }

  class BitVector;
  /**
    * Calculates a gauss elimation over matrix <code>v</code>.
    *
    * <em>Note: lines with only "0" will be erased.
    *
    * @param the matrix v
    * @param posy starting line (for recursion)
    * @param posy starting column (for recursion)
    *
    * @return rank of the new matrix.
    */
  int gauss(list<BitVector>& v, list<BitVector>::iterator posy, int posx);

  /**
    * Calculates a gauss elimation over matrix <code>v</code> and
    * leads to the reduced row echelon form of the matrix.
    *
    * <em>Note: lines with only "0" will be erased.
    *
    * @param the matrix v
    * @param posy starting line (for recursion)
    * @param posy starting column (for recursion)
    *
    * @return rank of the reduced row echelon form of the matrix.
    */
  int gaussJordan(list<BitVector>& v, list<BitVector>::iterator pos, int posx);


//  void swap(list<BitVector>::iterator first, list<BitVector>::iterator second)


} // namespace graphlib
#endif // GRAPHLIB_UTILITIES_H

/* end of file */
