/***************************************************************************
                          node.cpp  -  description
                             -------------------
    begin                : Thu Mar 1 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "node.h"
#include "nodedata.h"
#include "coordinates.h"
#include "edgedata.h"
#include "edgeid.h"

#include "trace.h"

#include <algorithm>

namespace graphlib {

//---------------------------------------------------------------------------
// Con-/Destructors
//---------------------------------------------------------------------------
Node::Node() : data_(0)
{ ; }

//---------------------------------------------------------------------------
// Operators
//---------------------------------------------------------------------------
bool Node::operator==(const Node& n) const
{
  return (data_ == n.data_);
}
bool Node::operator!=(const Node& n) const
{
  return (data_ != n.data_);
}
bool Node::operator<(const Node& n) const
{
  return (data()->id < n.data()->id);
}
bool Node::operator>(const Node& n) const
{
  return (data()->id > n.data()->id);
}

//---------------------------------------------------------------------------
// Input / Output
//---------------------------------------------------------------------------
ostream& operator<<(ostream& os, const Node& n)
{
  os << " "  << n.id() << " ";
//  os << "(" << n.data << ")";
//  constEdgeIter it;
//  for (it=n.data()->edges.begin(); it!=n.data()->edges.end();it++)
//    os << "       ["<< *it << "]";

  return os;
}

//---------------------------------------------------------------------------
// Information
//---------------------------------------------------------------------------
int Node::id() const
{
  return data()->id;
}

constNodeIter Node::pos() const
{
  return data()->pos;
}

unsigned Node::degree() const
{
  return data()->degree;
}

string Node::label() const
{
  return data()->label;
}

Coordinates Node::coord() const
{
  return data()->coord;
}

const Node& Node::opposite(const Edge& e) const
{
  const Node& s = *(e.data()->nodes.begin());

  if (*this == s)   // if this node == source of e
    return *(++(e.data()->nodes.begin()));
  else              // else
    return s;
}

bool Node::isNULL() const
{
  return (data_ == 0 ? true : false);
}

bool Node::isTarget(int source) const
{
  constEdgeIter edge;
  edge = find_if(data()->edges.begin(),data()->edges.end(),
                 EdgeSourceTarget(source,data()->id));

  // exists an edge (source,this)
  if (edge == data()->edges.end())
    return false;
  // wrong direction ?
  if (edge->target().id() == source)
    return false;

  return true;
}

constEdgeIter Node::findEdge(const Node& node) const
{
  return find_if(data()->edges.begin(),data()->edges.end(),
    EdgeSourceTarget(id(),node.id()));
}

//---------------------------------------------------------------------------
// Set Funtions
//---------------------------------------------------------------------------

constNodeIter Node::label(string label)
{
  data()->label = label;
  return data()->pos;
}
constNodeIter Node::coord(Coordinates coord)
{
  data()->coord = coord;
  return data()->pos;
}

//---------------------------------------------------------------------------
// Iterators
//---------------------------------------------------------------------------
constEdgeIter Node::adjBegin() const
{
  return data()->edges.begin();
}

constEdgeIter Node::adjEnd() const
{
  return data()->edges.end();
}

//---------------------------------------------------------------------------
// Private Functions
//---------------------------------------------------------------------------
const NodeData* Node::data() const throw(EmptyNodeError)
{
  if (!data_)
    THROW(EmptyNodeError,"Node::data() const")
  return data_;
}

NodeData* Node::data() throw(EmptyNodeError)
{
  if (!data_)
    THROW(EmptyNodeError,"Node::data()")
  return data_;
}

} // namespace graphlib

/* end of file */
