/***************************************************************************
                          prototype.h  -  description                              
                             -------------------                                         
    begin                : Thu Dec 30 1999                                           
    copyright            : (C) 1999 by Gleiss Petra                         
    email                : petra@gleiss.at                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef PROTOTYPE_H
#define PROTOTYPE_H

#include "graphlib.h"
#include "cycle.h"

#include <iostream.h>
#include <iomanip.h>


/**
  * @Short A prototype Cycle.
  *
  * cycles: prototypes will be generated from 3 or 4 nodes:
  *            odd (r,p,q)    == (r,p,q,INF,INF)
  *      even (r,p,q,x) == (r,p,q,x,INF)
  *
  * u-pathes: prototpyes will be generated from 4 or 5 nodes:
  *     odd (r,p,q,x,y)    == (INF,u,v,x,y)
  *     even (r,p,q,x,y) == (w,u,v,x,y)
  *
  *     p -.-.-.- x - y -.-.-.- q
  *     p -.-.- x - r - y -.-.- q
  */

namespace graphlib {

class Prototype : public Cycle
{
    //------------------------------------------------------ friend functions
    /**
      * Output-operator.
      *
      * @param ostream&
      * @param const Prototype&
      * @return ostream&
      */
    friend ostream& operator<< (ostream&, const Prototype&);

  public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Constructor.
      */
    Prototype(unsigned asize=0, const Graph* g=0);

    /**
      * Constructor.
      */
    Prototype(const Cycle&);

    /**
      * Copy-Constructor.
      */
    Prototype(const Prototype&);

    /**
      * Destructor.
      */
    virtual ~Prototype() { ; }

    //------------------------------------------------------ Operators
    /**
      * Assignment Operator.
      */
    Prototype& operator=(const Prototype&);

    //------------------------------------------------------ Information
    /**
      * Sets the nodenumbers needed for calcucating the relevant cycles
      * from the prototypes.
      *
      * @param r
      * @param p
      * @param q
      * @param x
      * @param y
      */
    Prototype& setNodes(int r=INFINITY, int p=INFINITY, int q=INFINITY,
                        int x=INFINITY, int y=INFINITY);

    /**
      * Returns r.
      */
    int r() const { return r_; }

    /**
      * Returns p.
      */
    int p() const { return p_; }

    /**
      * Returns q.
      */
    int q() const { return q_; }

    /**
      * Returns x.
      */
    int x() const { return x_; }

    /**
      * Returns y.
      */
    int y() const { return y_; }

  private:
    //------------------------------------------------------ data
    int r_;   // )  nodenumbers to calculate
    int p_;   // )    relevant cycles from protocycles
    int q_;   // )      (odd cycles: r,p,q; x=-1) (even cycles: r,p,q,x)
    int x_;   // )    relevant u-pathes form protopathes
    int y_;   // )      (odd u-pathes: r,p,q,x) (even u-pathes: r,p,q,x,y)
  // private functions


}; // class Prototype

} // namespace graphlib

#endif // PROTOTYPE_H

/* end of file */
