/***************************************************************************
                          subgraph.h  -  description
                             -------------------
    begin                : Thu June 29 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SUBGRAPH_H
#define SUBGRAPH_H

#include "basisgraph.h"
#include "graph.h"
#include "graphlib_error.h"
using namespace graphlibError;

#include <string>
#include <vector>
#include <list>
#include <iostream>

namespace graphlib {

/**
  *@short A subgraph of an undirected graph
  *
  * A subgraph <var>G'=(V',E')</var> of a graph <var>G=(V,E)</var>
  * consists of a set of nodes <var>V'</var> contained in <var>V</var>
  * and a set of edges <var>E'</var> contained in <var>E</var>.
  * The nodes and edges of the subgraphs are the same as in the graph.
  * The "owner" of the nodes and edges is the graph <var>G</var>.
  *
  * @see Edge
  * @see Node
  * @see Graph
  */

class SubGraph : virtual public BasisGraph
{
  public:
    //------------------------------------------------------ Con-/Destructors
    /**
      * Constructor.
      * Generates an empty SubGraph, without any nodes and any edges,
      * which belongs to <code>graph</code>, with number <code>n</code>
      */
    SubGraph(const Graph* graph=0, int nr=0);

    /**
      * Destructor. Nothing to do.
      */
    virtual ~SubGraph() { ; }

    /**
      * Copy Constructor.
      * Copies the list of nodes and edges, but they still belongs to
      * <var>graph</var>.
      */
    SubGraph(const SubGraph&);

    //------------------------------------------------------ Operators
    /**
      * Assignement Operator
      */
    SubGraph& operator=(const SubGraph& g);

    //------------------------------------------------------ Operators
    /**
      * Adds the <code>node</code> to <var>nodes</var>, if it does not
      * already exists, otherwise throw a DoubleError.
      */
    void add(const Node& node) throw(DoubleError,EmptyNodeError);

    /**
      * Adds the <code>edge</code> to <var>edges</var>, if it does not
      * already exists, otherwise throw a DoubleError and if the
      * source and target of <code>edge</code> is contained in <var>nodes</var>,
      * otherwise throw a EmptyNodeError.
      */
    void add(const Edge& edge)
        throw(DoubleError,EmptyNodeError,EmptyEdgeError);

  protected:
    //------------------------------------------------------ Name
    /// graph of which this one is a subgraph
    const Graph*      graph;

};  // class SubGraph

} // namespace graphlib

#endif // SUBGRAPH_H

/* end of file */
