/***************************************************************************
                          utilities.h  -  description
                             -------------------
    begin                : Tue Mar 13 2001
    copyright            : (C) 2001 by Gleiss Petra Manuela
    email                : petra@gleiss.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef UTILITIES_H
#define UTILITIES_H

#include <cstdlib>
#include <ctime>

#include <string>

#include <strstream>
#include <iomanip>

#include <vector>
#include <list>
#include <map>


  /**
    * Output-operator.
    */
  template<class T> inline
  ostream& operator<<(ostream& os, const vector<T>& vc)
  {
    copy(vc.begin(),vc.end(),ostream_iterator<T>(os," "));
    return os;
  }

  /**
    * Output-operator.
    */
  template<class T> inline
  ostream& operator<<(ostream& os, const list<T>& vc)
  {
    copy(vc.begin(),vc.end(),ostream_iterator<T>(os," "));
    return os;
  }

  /**
    * Output-operator.
    */
  template<class A, class B> inline
  ostream& operator<<(ostream& os, const map<A,B>& amap)
  {
    map<A,B>::const_iterator it=amap.begin();
    map<A,B>::const_iterator end=amap.end();

    while (it != end)
    {
      os << it->first << " <=> " << it->second << endl;
      it++;
    }

    return os;
  }


  /**
    * Returns the <code>arg</code> as string.
    */
  template <typename T> inline
  string Ttos(const T& arg)
  {
    ostrstream buffer;
    buffer << arg << ends;
    string tmp(buffer.str());
    buffer.rdbuf()->freeze(false);
    return tmp;
  }

  /**
    * Returns the adsolute difference between <code>a</code>
    * and <code>b</code>.
    */
  template <typename T> inline
  T delta(const T& a, const T& b)
  {
    if (a < b)
      return (b-a);
    else
      return (a-b);
  }

  /**
    * Returns the sign of <code>x</code>,
    */
  template <typename T> inline
  int sign(const T& x)
  {
    if (x > 0)
      return 1;
    if (x < 0)
      return -1;
    return 0;
  }

  /**
    * Returns the localtime as string.
    * Format: e.g. Tue Feb  5 20:45:24 2002
    */
  inline string timeStamp()
  {
    time_t datum=time(NULL);
    return string(asctime(localtime(&datum)));
  }

  /**
    * Returns a pseudo-random number between 0.0 and 1.0
    */
  inline double urn()
  {
    return ((double) rand())/RAND_MAX;
  }

  /**
    * Returns sting = fname.number (number: xxx)
    */
  inline string getname(const string& fname, int number)
  {
    ostrstream buffer;
    buffer << fname << ".";
    buffer << (number/100);
    number %= 100;
    buffer << (number/10);
    number %= 10;
    buffer << number << ends;
    string tmp(buffer.str());
    buffer.rdbuf()->freeze(false);
    return tmp;
  }

#endif // UTILITIES_H

/* end of file */
